library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
library work;
use work.common.all;

entity cf_branch_predictor is
    generic (
		reset_vec : std_ulogic_vector(31 downto 0) := 32x"80000000"
	);
	port (
		i_clk : in std_ulogic;
		i_clk_enable : in std_ulogic := '1';
		i_reset_n : in std_ulogic := '1';
		
		i_predicted_miss : in boolean;
		i_compressed : in boolean;
		i_next_pc : in std_ulogic_vector(31 downto 0);
		i_mem_req_fire : in boolean;
		o_predicted_pc : out std_ulogic_vector(31 downto 0)
	);
end entity; -- cf_branch_predictor

architecture arch of cf_branch_predictor is
	type StateType is record
		next_pc : std_ulogic_vector(31 downto 0);
	end record;

	constant Offset_Half : integer := 2;
	constant Offset_Full : integer := 4;

	signal s_r : StateType;
	signal s_r_next : StateType;
begin
	
	process (all)
		variable v_r : StateType;
		variable v_next_pc : std_ulogic_vector(31 downto 0);
	begin
		v_r := s_r;

		if i_predicted_miss then
			v_next_pc := i_next_pc;
		elsif i_compressed then
			v_next_pc := std_ulogic_vector(unsigned(s_r.next_pc) + Offset_Half);
		else
			v_next_pc := std_ulogic_vector(unsigned(s_r.next_pc) + Offset_Full);
		end if;

		if i_mem_req_fire then
			v_r.next_pc := v_next_pc;
		end if;

		if i_reset_n = '0' then
			v_r.next_pc := std_ulogic_vector(unsigned(reset_vec) - Offset_Full);
		end if;
		
		s_r_next <= v_r;
		o_predicted_pc <= v_next_pc;
	end process;
	
	process
	begin
		wait until rising_edge(i_clk) and i_clk_enable = '1';
		
		s_r <= s_r_next;
	end process;
	
end architecture;