library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity cf_dualport_ram is
    generic (
		addr_width : natural := 8;
		data_width : natural := 8
	);
	port (
		i_clk : in std_ulogic;
		i_clk_enable : in std_ulogic := '1';
		i_reset_n : in std_ulogic := '1';
		i_addr_a : in std_ulogic_vector(addr_width - 1 downto 0);
		i_addr_b : in std_ulogic_vector(addr_width - 1 downto 0);
		i_write_enable_a : in boolean := false;
		i_write_enable_b : in boolean := false;
		i_data_a : in std_ulogic_vector(data_width - 1 downto 0) := (others => '0');
		i_data_b : in std_ulogic_vector(data_width - 1 downto 0) := (others => '0');
		o_data_a : out std_ulogic_vector(data_width - 1 downto 0);
		o_data_b : out std_ulogic_vector(data_width - 1 downto 0)
	);
end entity; -- cf_dualport_ram

architecture arch of cf_dualport_ram is
	subtype element_type is std_ulogic_vector(data_width - 1 downto 0);
	type memory_type is array(0 to (2 ** addr_width) - 1) of element_type;

	shared variable v_memory : memory_type;
begin

	-- Port A
	process
	begin
		wait until rising_edge(i_clk) and i_clk_enable = '1';

		if i_write_enable_a then
			v_memory(to_integer(unsigned(i_addr_a))) := i_data_a;
		end if;
		
		o_data_a <= v_memory(to_integer(unsigned(i_addr_a)));
	end process;
	
	-- Port B
	process
	begin
		wait until rising_edge(i_clk) and i_clk_enable = '1';
		
		if i_write_enable_b then
			v_memory(to_integer(unsigned(i_addr_b))) := i_data_b;
		end if;
		
		o_data_b <= v_memory(to_integer(unsigned(i_addr_b)));
	end process;

end architecture;
