library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity cf_intreg is
	port (
		i_clk : in std_ulogic;
		i_clk_enable : in std_ulogic := '1';
		i_reset_n : in std_ulogic := '1';
		i_read_idx_a : in std_ulogic_vector(4 downto 0);
		i_read_idx_b : in std_ulogic_vector(4 downto 0);
		i_write_idx : in std_ulogic_vector(4 downto 0);
		i_write_data : in std_ulogic_vector(31 downto 0);
		i_write_enable : in boolean := false;
		o_a_q : out std_ulogic_vector(31 downto 0);
		o_b_q : out std_ulogic_vector(31 downto 0)
	);
end entity; -- cf_intreg

architecture arch of cf_intreg is
	type RegisterArrayType is array(1 to 31) of std_ulogic_vector(31 downto 0);
	
	type StateType is record
		registers : RegisterArrayType;
	end record;

	signal s_r : StateType;
	signal s_r_next : StateType;
begin
	
	process (all)
		variable v_r : StateType;
		variable v_read_idx_a : natural;
		variable v_read_idx_b : natural;
		variable v_write_idx : natural;
	begin
		v_r := s_r;
		
		v_read_idx_a := to_integer(unsigned(i_read_idx_a));
		v_read_idx_b := to_integer(unsigned(i_read_idx_b));
		v_write_idx := to_integer(unsigned(i_write_idx));
		
		if i_write_enable and (v_write_idx /= 0) then
			v_r.registers(v_write_idx) := i_write_data;
		end if;
		
		if i_reset_n = '0' then
			v_r.registers := (others => (others => '0'));
			v_read_idx_a := 0;
			v_read_idx_b := 0;
		end if;
		
		if v_read_idx_a = 0 then
			o_a_q <= (others => '0');
		else
			o_a_q <= s_r.registers(v_read_idx_a);
		end if;
		
		if v_read_idx_b = 0 then
			o_b_q <= (others => '0');
		else
			o_b_q <= s_r.registers(v_read_idx_b);
		end if;
	
		s_r_next <= v_r;
	end process;
	
	process
	begin
		wait until rising_edge(i_clk) and i_clk_enable = '1';
		
		s_r <= s_r_next;
	end process;
	
end architecture;