library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity cf_multiplier is
	port (
		i_a : in std_ulogic_vector(31 downto 0);
		i_b : in std_ulogic_vector(31 downto 0);
		i_unsigned_op : in boolean;
		i_mixed_op : in boolean;
		o_q : out std_ulogic_vector(63 downto 0)
	);
end entity; -- cf_multiplier

architecture arch of cf_multiplier is
begin
	
	process (all)
		variable v_a : signed(32 downto 0);
		variable v_b : signed(32 downto 0);
		variable v_q : signed(65 downto 0);
	begin
		v_a(31 downto 0) := signed(i_a);
		v_b(31 downto 0) := signed(i_b);

		if i_mixed_op then
			v_a(32) := v_a(31);
			v_b(32) := '0';
		elsif i_unsigned_op then
			v_a(32) := '0';
			v_b(32) := '0';
		else
			v_a(32) := v_a(31);
			v_b(32) := v_b(31);
		end if;

		v_q := v_a * v_b;

		o_q <= std_ulogic_vector(v_q(63 downto 0));
	end process;
	
end architecture;