library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.common.all;

entity tb_data_mem_align is
end entity ; -- tb_data_mem_align

architecture arch of tb_data_mem_align is
	procedure wait_for_result(signal s_clk : in std_ulogic; signal s_rsp_valid : in boolean) is
	begin
		wait until rising_edge(s_clk);

		while not s_rsp_valid loop
			wait until rising_edge(s_clk);
		end loop;
	end procedure;

	procedure wait_for_complete(signal s_clk : in std_ulogic; signal s_req_ready : in boolean) is
	begin
		wait until rising_edge(s_clk);
		
		while not s_req_ready loop
			wait until rising_edge(s_clk);
		end loop;
	end procedure;

	constant CLK_PERIOD : time := 1 ns;
	
	signal s_clk : std_ulogic := '0';
	signal s_reset_n : std_ulogic := '0';
	signal s_simulation_stop : boolean := false;
	
	signal s_mem_in : MemoryInput;
	signal s_mem_out : MemoryOutput;
	signal s_dca_req_valid : boolean;
	signal s_dca_req_ready : boolean;
	signal s_dca_rsp_valid : boolean;
	signal s_dca_rsp_ready : boolean;
	signal s_dca_write_enable : boolean;
	signal s_dca_cache_enable : std_ulogic;
	signal s_dca_exception : boolean;
	signal s_dca_size : std_ulogic_vector(1 downto 0);
	signal s_dca_addr_in : std_ulogic_vector(31 downto 0);
	signal s_dca_addr_out : std_ulogic_vector(31 downto 0);
	signal s_dca_data_in : std_ulogic_vector(31 downto 0);
	signal s_dca_data_out : std_ulogic_vector(31 downto 0);
begin
	
    ram: entity work.tb_ram
		generic map (
			addr_width => 5,
			init_file => "cache.hex",
			req_count => 0,
			rsp_count => 0
		)
		port map (
			i_clk => s_clk,
			i_reset_n => s_reset_n,
			i_in => s_mem_in,
			o_out => s_mem_out
		);
	
    dcache: entity work.cf_data_mem_align
		port map (
			i_clk => s_clk,
			i_reset_n => s_reset_n,
			i_addr => s_dca_addr_out,
			i_size => s_dca_size,
			o_addr => s_dca_addr_in,
			o_data => s_dca_data_in,
			i_data => s_dca_data_out,
			i_write_enable => s_dca_write_enable,
			i_cache_enable => s_dca_cache_enable,
			i_req_valid => s_dca_req_valid,
			o_req_ready => s_dca_req_ready,
			o_rsp_valid => s_dca_rsp_valid,
			i_rsp_ready => s_dca_rsp_ready,
			o_exception => s_dca_exception,
			i_mem_out => s_mem_out,
			o_mem_in => s_mem_in
		);
	
	process
	begin
		while not s_simulation_stop loop
			s_clk <= not s_clk;
			wait for CLK_PERIOD / 2;
		end loop;
		wait;
	end process;
	
	process
	begin
		s_dca_cache_enable <= '0';
		s_dca_rsp_ready <= true;
		s_dca_write_enable <= false;
		s_dca_size <= "10";
		s_dca_data_out <= (others => '0');
		
		s_reset_n <= '0';
		wait until rising_edge(s_clk);
		
		s_reset_n <= '1';
		wait until rising_edge(s_clk);

		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"0";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_cache_enable <= '1';
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"8";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= false;
		s_dca_cache_enable <= '0';
		wait until rising_edge(s_clk);

		s_dca_write_enable <= true;
		s_dca_data_out <= 32x"ABCDEF12";
		s_dca_size <= "10";
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"6";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_write_enable <= false;
		s_dca_size <= "10";
		s_dca_req_valid <= false;
		s_dca_addr_out <= 32x"0";
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"4";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_addr_out <= 32x"E";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= false;
		s_dca_addr_out <= 32x"0";
		wait until rising_edge(s_clk);
		
		s_dca_cache_enable <= '1';
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"E";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= false;
		s_dca_addr_out <= 32x"0";
		wait until rising_edge(s_clk);
		
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"E";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"10";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"12";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"14";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"16";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"1A";
		
		wait_for_complete(s_clk, s_dca_req_ready);
		
		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"1E";
		
		wait_for_complete(s_clk, s_dca_req_ready);

		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"F2";
		s_dca_write_enable <= true;
		
		wait_for_complete(s_clk, s_dca_req_ready);

		s_dca_req_valid <= true;
		s_dca_addr_out <= 32x"10";
		s_dca_write_enable <= false;
		
		wait_for_complete(s_clk, s_dca_req_ready);

		s_dca_req_valid <= false;
		s_dca_addr_out <= 32x"0";
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);

		wait;
	end process;
	
	process
	begin
		while s_reset_n = '0' loop
			wait until rising_edge(s_clk);
		end loop;
	
		while not s_dca_rsp_valid loop
			wait until rising_edge(s_clk);
		end loop;
		
		assert s_dca_addr_in = 32x"00000000" report "1) expect 00000000h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00600093" report "1) expect 00600093h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "1) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"00000008" report "2) expect 00000008h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"002081B3" report "2) expect 002081B3h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "2) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);

		assert s_dca_addr_in = 32x"00000006" report "3) expect 00000006h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert not s_dca_exception report "2a) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"00000004" report "4) expect 00000004h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"0020ABCD" report "4) expect 0020ABCDh data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "4) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"0000000E" report "5) expect 0000000Eh addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00940050" report "5) expect 00940050h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "5) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"0000000E" report "6) expect 0000000Eh addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00940050" report "6) expect 00940050h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "6) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"0000000E" report "7) expect 0000000Eh addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00940050" report "7) expect 00940050h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "7) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"00000010" report "8) expect 00000010h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00400094" report "8) expect 00400094h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "8) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"00000012" report "9) expect 00000012h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00950040" report "9) expect 00950040h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "9) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"00000014" report "10) expect 00000014h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00300095" report "10) expect 00300095h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "10) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"00000016" report "11) expect 00000016h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00960030" report "11) expect 00960030h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "11) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"0000001A" report "12) expect 0000001Ah addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00970020" report "12) expect 00970020h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "12) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"0000001E" report "13) expect 00000093h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00980010" report "13) expect 00980010h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "13) expect no exception" severity error;
		
		wait_for_result(s_clk, s_dca_rsp_valid);
		
		assert s_dca_addr_in = 32x"000000F2" report "14) expect 000000F0h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_exception report "14) expect exception" severity error;

		wait_for_result(s_clk, s_dca_rsp_valid);

		assert s_dca_addr_in = 32x"00000010" report "15) expect 00000010h addr: " & to_hstring(s_dca_addr_in) severity error;
		assert s_dca_data_in = 32x"00400094" report "15) expect 00400094h data: " & to_hstring(s_dca_data_in) severity error;
		assert not s_dca_exception report "15) expect no exception" severity error;

		wait until rising_edge(s_clk);

		report "All Tests performed" severity note;
		s_simulation_stop <= true;
		wait;
	end process;

end architecture;