library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.common.all;

entity tb_divider is
end entity ; -- tb_divider

architecture arch of tb_divider is
	constant CLK_PERIOD : time := 1 ns;
	
	signal s_clk : std_ulogic := '0';
	signal s_simulation_stop : boolean := false;
	
	signal s_dd : std_ulogic_vector(31 downto 0);
	signal s_dr : std_ulogic_vector(31 downto 0);
	signal s_q : std_ulogic_vector(31 downto 0);
	signal s_rem : std_ulogic_vector(31 downto 0);
	signal s_reset_n : std_ulogic := '0';
	signal s_unsigned_op : boolean := false;
	signal s_busy : boolean;
	signal s_enable : boolean;
begin
	
    div: entity work.cf_divider
		port map (
			i_clk => s_clk,
			i_reset_n => s_reset_n,
			i_dd => s_dd,
			i_dr => s_dr,
			i_enable => s_enable,
			i_unsigned_op => s_unsigned_op,
			o_busy => s_busy,
			o_q => s_q,
			o_rem => s_rem
		);
	
	process
	begin
		while not s_simulation_stop loop
			s_clk <= not s_clk;
			wait for CLK_PERIOD / 2;
		end loop;
		wait;
	end process;

	process
	begin
		s_reset_n <= '0';
		wait until rising_edge(s_clk);

		s_reset_n <= '1';
		wait until rising_edge(s_clk);

		s_dd <= std_logic_vector(to_unsigned(169, 32));
		s_dr <= std_logic_vector(to_unsigned(30, 32));
		s_unsigned_op <= true;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		s_dd <= std_logic_vector(to_unsigned(127, 32));
		s_dr <= std_logic_vector(to_unsigned(0, 32));
		s_unsigned_op <= true;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		s_dd <= std_logic_vector(to_unsigned(127, 32));
		s_dr <= std_logic_vector(to_unsigned(255, 32));
		s_unsigned_op <= true;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		s_dd <= std_logic_vector(to_unsigned(1023, 32));
		s_dr <= std_logic_vector(to_unsigned(128, 32));
		s_unsigned_op <= true;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		s_dd <= std_logic_vector(to_unsigned(1024, 32));
		s_dr <= std_logic_vector(to_unsigned(128, 32));
		s_unsigned_op <= true;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		s_dd <= std_logic_vector(to_signed(8, 32));
		s_dr <= std_logic_vector(to_signed(-2, 32));
		s_unsigned_op <= false;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		s_dd <= std_logic_vector(to_signed(-8, 32));
		s_dr <= std_logic_vector(to_signed(2, 32));
		s_unsigned_op <= false;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		s_dd <= std_logic_vector(to_signed(-8, 32));
		s_dr <= std_logic_vector(to_signed(-2, 32));
		s_unsigned_op <= false;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		s_dd <= std_logic_vector(to_signed(-2147483648, 32));
		s_dr <= std_logic_vector(to_signed(-1, 32));
		s_unsigned_op <= false;
		s_enable <= true;
		wait until rising_edge(s_clk);
		
		s_enable <= false;
		wait until rising_edge(s_clk);

		while s_busy loop
			wait until rising_edge(s_clk);
		end loop;

		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);

		/*
		s_a <= 32x"19";
		s_b <= 32x"2";
		s_divide <= false;
		s_unsigned_op <= true;
		wait until rising_edge(s_clk);

		s_a <= 32x"19";
		s_b <= std_ulogic_vector(to_signed(-4, 32));
		s_divide <= true;
		s_unsigned_op <= false;
		wait until rising_edge(s_clk);

		s_a <= 32x"19";
		s_b <= std_ulogic_vector(to_signed(-4, 32));
		s_divide <= false;
		s_unsigned_op <= false;
		wait until rising_edge(s_clk);
		*/

		report "All Tests performed" severity note;
		s_simulation_stop <= true;
		wait;
	end process;

end architecture;