library ieee;
use ieee.std_logic_1164.all;
library work;
use work.common.all;

entity tb_execute is
end entity ; -- tb_execute

architecture arch of tb_execute is
	constant clk_period : time := 1 ns;
	
	signal s_clk : std_ulogic := '0';
	signal s_hold : std_ulogic := '0';
	signal s_valid : std_ulogic := '0';
	signal s_reset_n : std_ulogic := '0';
	signal s_simulation_stop : boolean := false;
	signal s_pc : std_ulogic_vector(31 downto 0) := (others => '0');
	signal s_instr : std_ulogic_vector(31 downto 0) := (others => '0');
	signal s_compressed : std_ulogic := '0';
	signal s_instr_vec : std_ulogic_vector(Instr_Total - 1 downto 0) := (others => '0');
	signal s_isa_type_vec : std_ulogic_vector(ISA_Total - 1 downto 0) := (others => '0');
	signal s_unsigned_op : std_ulogic := '0';
--	signal s_exception : std_ulogic := '0';
	signal s_next_pc : std_ulogic_vector(31 downto 0);
	signal s_srcreg1_data : std_ulogic_vector(31 downto 0) := (others => '0');
	signal s_srcreg2_data : std_ulogic_vector(31 downto 0) := (others => '0');
	signal s_srcreg1_idx : std_ulogic_vector(4 downto 0);
	signal s_srcreg2_idx : std_ulogic_vector(4 downto 0);
	signal s_dstreg_idx : std_ulogic_vector(4 downto 0);
	signal s_data : std_ulogic_vector(31 downto 0);
	signal s_mem_addr : std_ulogic_vector(31 downto 0);
	signal s_mem_read : std_ulogic;
	signal s_mem_write : std_ulogic;
	signal s_mem_size : std_ulogic_vector(1 downto 0);
	signal s_mem_sign : std_ulogic;
	signal s_valid_out : std_ulogic;
begin
	execute: entity work.cf_execute
		port map (
			i_clk => s_clk,
			i_hold => s_hold,
			i_reset_n => s_reset_n,
			i_valid => s_valid,
			i_pc => s_pc,
			i_instr => s_instr,
			i_compressed => s_compressed,
			i_instr_vec => s_instr_vec,
			i_isa_type_vec => s_isa_type_vec,
			i_unsigned_op => s_unsigned_op,
			i_interrupt_enable => '0',
			i_mtvec => (others => '0'),
			i_mepc => (others => '0'),
			i_sepc => (others => '0'),
			i_mode => (others => '0'),
			i_dec_exception => '0',
			i_dec_error_code => (others => '0'),
			i_mem_exception => '0',
			i_mem_error_code => (others => '0'),
			i_mem_error_pc => (others => '0'),
			i_srcreg1_data => s_srcreg1_data,
			i_srcreg2_data => s_srcreg2_data,
			o_srcreg1_idx => s_srcreg1_idx,
			o_srcreg2_idx => s_srcreg2_idx,
			o_dstreg_idx => s_dstreg_idx,
			i_hazard1_idx => (others => '0'),
			i_hazard2_idx => (others => '0'),
			i_hazard_csr => '0',
			o_q => s_data,
			o_next_pc => s_next_pc,
			o_mem_addr => s_mem_addr,
			o_mem_read => s_mem_read,
			o_mem_write => s_mem_write,
			o_mem_size => s_mem_size,
			o_mem_sign => s_mem_sign,
			o_valid => s_valid_out
		);
	
	process
	begin
		while not s_simulation_stop loop
			s_clk <= not s_clk;
			wait for clk_period / 2;
		end loop;
		wait;
	end process;
	
	process
	begin
		s_reset_n <= '0';
		wait until rising_edge(s_clk);
		s_reset_n <= '1';
		
		s_pc <= 32x"0";
		s_instr <= b"000000000100_00010_000_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_Add) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"5";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"4" report "1) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "1) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "1) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"9" report "1) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "1) mem read" severity error;
		assert s_mem_write = '0' report "1) mem write" severity error;
		
		s_pc <= 32x"4";
		s_instr <= b"0000000_00100_00011_000_00010_011_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_Add) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_R_Type) <= '1';
		s_srcreg1_data <= 32x"5";
		s_srcreg2_data <= 32x"11";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"8" report "2) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00010" report "2) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00011" report "2) source reg 1 idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_srcreg2_idx = b"00100" report "2) source reg 2 idx " & to_bstring(s_srcreg2_idx) severity error;
		assert s_data = 32x"16" report "2) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "2) mem read" severity error;
		assert s_mem_write = '0' report "2) mem write" severity error;
		
		s_pc <= 32x"8";
		s_instr <= b"0100000_00100_00011_000_00010_011_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_Sub) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_R_Type) <= '1';
		s_srcreg1_data <= 32x"5";
		s_srcreg2_data <= 32x"11";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"c" report "3) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00010" report "3) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00011" report "3) source reg 1 idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_srcreg2_idx = b"00100" report "3) source reg 2 idx " & to_bstring(s_srcreg2_idx) severity error;
		assert s_data = 32x"FFFFFFF4" report "3) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "3) mem read" severity error;
		assert s_mem_write = '0' report "3) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"100000000100_00010_010_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_SetLessThanSigned) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"FFFFF803";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "4) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "4) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "4) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"1" report "4) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "4) mem read" severity error;
		assert s_mem_write = '0' report "4) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"100000000100_00010_010_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_SetLessThanSigned) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"0";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "5) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "5) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "5) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"0" report "5) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "5) mem read" severity error;
		assert s_mem_write = '0' report "5) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"100000000100_00010_011_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_SetLessThanUnsigned) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"FFFFF803";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "6) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "6) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "6) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"1" report "6) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "6) mem read" severity error;
		assert s_mem_write = '0' report "6) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"100000000100_00010_011_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_SetLessThanUnsigned) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"0";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "7) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "7) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "7) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"1" report "7) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "7) mem read" severity error;
		assert s_mem_write = '0' report "7) mem write" severity error;

		s_pc <= 32x"c";
		s_instr <= b"100000000100_00010_111_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_And) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"1000";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "8) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "8) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "8) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"1000" report "8) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "8) mem read" severity error;
		assert s_mem_write = '0' report "8) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"000000000100_00010_110_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_Or) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"1";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "9) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "9) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "9) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"5" report "9) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "9) mem read" severity error;
		assert s_mem_write = '0' report "9) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"100000000100_00010_100_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_Xor) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"FFFFFFFF";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "10) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "10) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "10) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = b"00000000000000000000011111111011" report "10) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "10) mem read" severity error;
		assert s_mem_write = '0' report "10) mem write" severity error;
		
		
		
		s_pc <= 32x"c";
		s_instr <= b"0000000_00101_00010_111_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_And) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_R_Type) <= '1';
		s_srcreg1_data <= 32x"1000";
		s_srcreg2_data <= 32x"1001";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "11) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "11) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "11) source reg 1 idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_srcreg2_idx = b"00101" report "11) source reg 2 idx " & to_bstring(s_srcreg2_idx) severity error;
		assert s_data = 32x"1000" report "11) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "11) mem read" severity error;
		assert s_mem_write = '0' report "11) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"0000000_00001_00010_110_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_Or) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_R_Type) <= '1';
		s_srcreg1_data <= 32x"1";
		s_srcreg2_data <= 32x"1000";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "12) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "12) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "12) source reg 1 idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_srcreg2_idx = b"00001" report "12) source reg 2 idx " & to_bstring(s_srcreg2_idx) severity error;
		assert s_data = 32x"1001" report "12) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "12) mem read" severity error;
		assert s_mem_write = '0' report "12) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"0000000_10100_00010_100_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_Xor) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_R_Type) <= '1';
		s_srcreg1_data <= 32x"FFFFFFFF";
		s_srcreg2_data <= 32x"1000";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "13) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "13) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "13) source reg 1 idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_srcreg2_idx = b"10100" report "13) source reg 2 idx " & to_bstring(s_srcreg2_idx) severity error;
		assert s_data = b"11111111111111111110111111111111" report "13) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "13) mem read" severity error;
		assert s_mem_write = '0' report "13) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"00000000000000001100_01001_011_0111";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_LoadUpperImm) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_U_Type) <= '1';
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "14) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"01001" report "14) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_data = b"00000000000000001100000000000000" report "14) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "14) mem read" severity error;
		assert s_mem_write = '0' report "14) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"00000000000000001100_01011_001_0111";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_AddUpperImmToPc) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_U_Type) <= '1';
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "15) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"01011" report "15) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_data = b"00000000000000001100000000001100" report "15) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "15) mem read" severity error;
		assert s_mem_write = '0' report "15) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"00000000001000000000_01011_110_1111";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_JumpAndLink) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_J_Type) <= '1';
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"e" report "16) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"01011" report "16) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_data = 32x"10" report "16) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "16) mem read" severity error;
		assert s_mem_write = '0' report "16) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"000000000100_01010_000_11111_110_0111";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_JumpAndLinkReg) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"10";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"20" report "17) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"11111" report "17) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"01010" report "17) source reg 1 idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"10" report "17) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "17) mem read" severity error;
		assert s_mem_write = '0' report "17) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"0_000000_00010_00001_000_0011_0_110_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_BranchEqual) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_B_Type) <= '1';
		s_srcreg1_data <= 32x"1";
		s_srcreg1_data <= 32x"2";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "18) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_srcreg1_idx = b"00001" report "18) source reg 1 idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_srcreg2_idx = b"00010" report "18) source reg 2 idx " & to_bstring(s_srcreg2_idx) severity error;
		assert s_mem_read = '0' report "18) mem read" severity error;
		assert s_mem_write = '0' report "18) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"0_000000_00010_00001_001_0011_0_110_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_BranchNotEqual) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_B_Type) <= '1';
		s_srcreg1_data <= 32x"1";
		s_srcreg1_data <= 32x"2";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"12" report "19) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_srcreg1_idx = b"00001" report "19) source reg 1 idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_srcreg2_idx = b"00010" report "19) source reg 2 idx " & to_bstring(s_srcreg2_idx) severity error;
		assert s_mem_read = '0' report "19) mem read" severity error;
		assert s_mem_write = '0' report "19) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"000000000010_00010_001_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_ShiftLeftLogic) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"5";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "30) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "30) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "30) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"14" report "30) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "30) mem read" severity error;
		assert s_mem_write = '0' report "30) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"000000000010_00010_001_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_ShiftRightLogic) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"80000000";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "31) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "31) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "31) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"20000000" report "31) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "31) mem read" severity error;
		assert s_mem_write = '0' report "31) mem write" severity error;
		
		s_pc <= 32x"c";
		s_instr <= b"000000000010_00010_001_00001_001_0011";
		s_instr_vec <= (others => '0');
		s_instr_vec(Instr_ShiftRightArith) <= '1';
		s_isa_type_vec <= (others => '0');
		s_isa_type_vec(ISA_I_Type) <= '1';
		s_srcreg1_data <= 32x"80000000";
		wait until rising_edge(s_clk);
		
		assert s_next_pc = 32x"10" report "32) next_pc " & to_hstring(s_next_pc) severity error;
		assert s_dstreg_idx = b"00001" report "32) destination reg idx " & to_bstring(s_dstreg_idx) severity error;
		assert s_srcreg1_idx = b"00010" report "32) source reg idx " & to_bstring(s_srcreg1_idx) severity error;
		assert s_data = 32x"E0000000" report "32) reg value " & to_bstring(s_data) severity error;
		assert s_mem_read = '0' report "32) mem read" severity error;
		assert s_mem_write = '0' report "32) mem write" severity error;

		s_simulation_stop <= true;
		wait;
	end process;

end architecture;