library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_fetch is
end entity ; -- tb_fetch

architecture arch of tb_fetch is
	constant clk_period : time := 1 ns;
	
	signal s_clk : std_ulogic := '0';
	signal s_hold : std_ulogic := '0';
	signal s_reset_n : std_ulogic := '0';
	signal s_mem_addr_in : std_ulogic_vector(31 downto 0);
	signal s_mem_addr_out : std_ulogic_vector(31 downto 0);
	signal s_mem_data : std_ulogic_vector(31 downto 0);
	signal s_mem_data_addr : std_ulogic_vector(31 downto 0);
	signal s_mem_addr_d : std_ulogic_vector(31 downto 0);
	signal s_mem_req_ready : std_ulogic := '0';
	signal s_mem_req_valid : std_ulogic;
	signal s_mem_rsp_valid : std_ulogic;
	signal s_mem_rsp_ready : std_ulogic;
	signal s_next_pc : std_ulogic_vector(31 downto 0);
	signal s_pc : std_ulogic_vector(31 downto 0) := (others => '0');
	signal s_simulation_stop : boolean := false;
begin
	fetch: entity work.cf_fetch
		port map (
			i_clk => s_clk,
			i_hold => s_hold,
			i_reset_n => s_reset_n,
			i_predicted_pc => (others => '0'),
			o_mem_req_valid => s_mem_req_valid,
			i_mem_req_ready => s_mem_req_ready,
			o_mem_addr => s_mem_addr_out,
			i_mem_rsp_valid => s_mem_rsp_valid,
			o_mem_rsp_ready => s_mem_rsp_ready,
			i_mem_data => s_mem_data,
			i_mem_addr => s_mem_addr_in,
			i_mem_exception => '0',
			i_next_pc => s_next_pc,
			o_pc => s_pc
		);
		
	s_next_pc <= (others => '0') when s_mem_rsp_valid = '0' else std_ulogic_vector(unsigned(s_pc) + 4);
	
	process
	begin
		while not s_simulation_stop loop
			s_clk <= not s_clk;
			wait for clk_period / 2;
		end loop;
		wait;
	end process;
	
	process
	begin
		s_reset_n <= '0';
		wait until rising_edge(s_clk);
		s_reset_n <= '1';
		
		s_mem_req_ready <= '0';
		s_mem_rsp_valid <= '0';
		s_mem_data <= (others => '0');
		wait until rising_edge(s_clk);
		
		s_mem_req_ready <= '1';
		wait until rising_edge(s_clk);
		
		while s_mem_req_valid = '0' loop
			wait until rising_edge(s_clk);
		end loop;
		
		s_mem_req_ready <= '0';
		s_mem_addr_d <= s_mem_addr_out;
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		
		s_mem_req_ready <= '1';
		s_mem_data <= x"1234ABCD";
		s_mem_addr_in <= s_mem_addr_d;
		s_mem_rsp_valid <= '1';
		wait until rising_edge(s_clk);
		
		while s_mem_req_valid = '0' loop
			wait until rising_edge(s_clk);
		end loop;
		
		s_mem_req_ready <= '0';
		s_mem_addr_d <= s_mem_addr_out;
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);

		s_mem_req_ready <= '1';
		s_mem_data <= x"ABCD0000";
		s_mem_addr_in <= s_mem_addr_d;
		s_mem_rsp_valid <= '1';
		wait until rising_edge(s_clk);
		
		while s_mem_req_valid = '0' loop
			wait until rising_edge(s_clk);
		end loop;
		
		s_mem_req_ready <= '0';
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);

		s_mem_addr_in <= s_mem_addr_out;	
		s_mem_req_ready <= '1';
		s_mem_data <= x"A1B2C3D4";
		s_mem_rsp_valid <= '1';
		wait until rising_edge(s_clk);

		s_simulation_stop <= true;
		wait;
	end process;

end architecture;