library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_intreg is
end entity ; -- tb_intreg

architecture arch of tb_intreg is
	constant clk_period : time := 1 ns;
	
	signal s_clk : std_ulogic := '0';
	signal s_reset_n : std_ulogic := '0';
	signal s_read_idx_a : std_ulogic_vector(4 downto 0) := (others => '0');
	signal s_read_idx_b : std_ulogic_vector(4 downto 0) := (others => '0');
	signal s_write_idx : std_ulogic_vector(4 downto 0) := (others => '0');
	signal s_dst_reg_data : std_ulogic_vector(31 downto 0);
	signal s_reg_a_q : std_ulogic_vector(31 downto 0);
	signal s_reg_b_q : std_ulogic_vector(31 downto 0);
	signal s_dst_write_enable : std_ulogic := '0';
	signal s_simulation_stop : boolean := false;
begin
	intreg: entity work.cf_intreg
		port map (
			i_clk => s_clk,
			i_reset_n => s_reset_n,
			i_read_idx_a => s_read_idx_a,
			i_read_idx_b => s_read_idx_b,
			i_write_idx => s_write_idx,
			i_write_data => s_dst_reg_data,
			i_write_enable => s_dst_write_enable,
			o_a_q => s_reg_a_q,
			o_b_q => s_reg_b_q
		);
	
	process
	begin
		while not s_simulation_stop loop
			s_clk <= not s_clk;
			wait for clk_period / 2;
		end loop;
		wait;
	end process;
	
	process
	begin
		s_reset_n <= '0';
		wait until rising_edge(s_clk);

		s_reset_n <= '1';
		wait until rising_edge(s_clk);
		
		s_read_idx_a <= 5x"1";
		wait until rising_edge(s_clk);
		
		assert s_reg_a_q = 32x"0" report "0) expect 0: " & to_hstring(s_reg_a_q) severity error;
		
		s_write_idx <= 5x"2";
		s_dst_reg_data <= 32x"42";
		s_dst_write_enable <= '1';
		wait until rising_edge(s_clk);
		
		s_read_idx_a <= 5x"2";
		s_dst_write_enable <= '0';
		wait until rising_edge(s_clk);
		
		assert s_reg_a_q = 32x"42" report "1) expect 42h: " & to_hstring(s_reg_a_q) severity error;

		s_simulation_stop <= true;
		wait;
	end process;

end architecture;