library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.common.all;

entity tb_mem_icache is
end entity ; -- tb_mem_cache

architecture arch of tb_mem_icache is
	constant CLK_PERIOD : time := 1 ns;
	
	signal s_clk : std_ulogic := '0';
	signal s_reset_n : std_ulogic := '0';
	signal s_simulation_stop : boolean := false;
	
	signal s_in : MemoryInput;
	signal s_out : MemoryOutput;
	signal s_mem_in : MemoryInput;
	signal s_mem_out : MemoryOutput;
	signal s_flush_cache : boolean;
	signal s_clk_counter : natural := 0;
begin
	
    ram: entity work.tb_ram
		generic map (
			addr_width => 6,
			init_file => "cache.hex",
			req_count => 0,
			rsp_count => 0
		)
		port map (
			i_clk => s_clk,
			i_reset_n => s_reset_n,
			i_in => s_mem_in,
			o_out => s_mem_out
		);
	
    cache: entity work.cf_mem_icache
		port map (
			i_clk => s_clk,
			i_reset_n => s_reset_n,
			i_sc_cnt => Invalid_SecurityContextCount,
			i_flush_cache => s_flush_cache,
			i_in => s_in,
			o_out => s_out,
			o_mem_in => s_mem_in,
			i_mem_out => s_mem_out,
			i_mem_sc_cnt => Invalid_SecurityContextCount,
			i_mem_paging_error => false
		);
	
	process
	begin
		while not s_simulation_stop loop
			s_clk <= not s_clk;
			wait for CLK_PERIOD / 2;
		end loop;
		wait;
	end process;

	process
	begin
		wait until rising_edge(s_clk);
		s_clk_counter <= s_clk_counter + 1;
	end process;

	process
	begin
		s_reset_n <= '0';
		wait until rising_edge(s_clk);

		s_reset_n <= '1';
		wait until rising_edge(s_clk);

		s_in.addr <= 32x"4";
		s_in.data <= 32x"0";
		s_in.mask <= "1111";
		s_in.req_valid <= true;
		s_in.rsp_ready <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= 32x"8";
		s_in.req_valid <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= 32x"4";
		s_in.req_valid <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= 32x"8";
		s_in.req_valid <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= 32x"24";
		s_in.req_valid <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= 32x"4";
		s_in.req_valid <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= 32x"28";
		s_in.req_valid <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= 32x"4";
		s_in.req_valid <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= 32x"48";
		s_in.req_valid <= true;

		wait until rising_edge(s_clk);

		while not s_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_in.req_valid <= false;

		s_in.addr <= (others => 'U');

		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);

		report "All Tests performed" severity note;
		s_simulation_stop <= true;
		wait;
	end process;

	s_flush_cache <= s_clk_counter = 9;

end architecture;