library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.common.all;

entity tb_mem_translation is
end entity ; -- tb_mem_translation

architecture arch of tb_mem_translation is
	constant CLK_PERIOD : time := 1 ns;
	
	signal s_clk : std_ulogic := '0';
	signal s_reset_n : std_ulogic := '0';
	signal s_simulation_stop : boolean := false;

	signal s_security_context : SecurityContext := (satp => (mode => '0', asid => (others => '0'), ppn => (others => '0')), sum => false, mxr => false, mprv => false, mode => Machine, mpp => Machine, cnt => First_SecurityContextCount);
	
	signal s_mem_in : MemoryInput;
	signal s_mem_out : MemoryOutput;

	signal s_mmt_in : MemoryInput;
	signal s_mmt_out : MemoryOutput;
begin
	
    ram: entity work.tb_ram
		generic map (
			addr_width => 6,
			init_file => "mem_translation.hex",
			req_count => 0,
			rsp_count => 0
		)
		port map (
			i_clk => s_clk,
			i_reset_n => s_reset_n,
			i_in => s_mem_in,
			o_out => s_mem_out
		);
	
    mmt: entity work.cf_mem_translation
    	generic map (
    		mem_accessor => Data
    	)
		port map (
			i_clk => s_clk,
			i_reset_n => s_reset_n,
			i_security_context => s_security_context,
			i_in => s_mmt_in,
			o_out => s_mmt_out,
			o_mem_in => s_mem_in,
			i_mem_out => s_mem_out
		);
	
	process
	begin
		while not s_simulation_stop loop
			s_clk <= not s_clk;
			wait for CLK_PERIOD / 2;
		end loop;
		wait;
	end process;
	
	process
	begin
		s_reset_n <= '0';
		wait until rising_edge(s_clk);
		
		s_reset_n <= '1';
		wait until rising_edge(s_clk);

		s_security_context.satp.mode <= '1';
		s_security_context.satp.ppn <= 22x"E0000";
		s_security_context.mode <= Supervisor;

		-- DataWrite succeeding
		s_mmt_in.addr <= x"1045201C";
		s_mmt_in.data <= x"00000042";
		s_mmt_in.mask <= "0001";
		s_mmt_in.write_enable <= true;
		s_mmt_in.req_valid <= true;
		wait until rising_edge(s_clk);

		while not s_mmt_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		-- DataRead succeeding
		s_mmt_in.addr <= x"10452018";
		s_mmt_in.write_enable <= false;
		s_mmt_in.req_valid <= true;
		wait until rising_edge(s_clk);

		while not s_mmt_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		-- InstructionFetch failing
		s_mmt_in.addr <= x"10452018";
		s_mmt_in.req_valid <= true;
		s_mmt_in.write_enable <= false;
		wait until rising_edge(s_clk);

		while not s_mmt_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		-- InstructionFetch succeeding
		s_mmt_in.addr <= x"10453018";
		s_mmt_in.req_valid <= true;
		s_mmt_in.write_enable <= false;
		wait until rising_edge(s_clk);

		while not s_mmt_out.req_ready loop
			wait until rising_edge(s_clk);
		end loop;

		s_mmt_in.req_valid <= false;
		wait until rising_edge(s_clk);

		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);

		report "All Tests performed" severity note;
		s_simulation_stop <= true;
		wait;
	end process;

	process
	begin
		s_mmt_in.rsp_ready <= true;

		while not s_mmt_out.rsp_valid loop
			wait until rising_edge(s_clk);
		end loop;

		wait until rising_edge(s_clk);

		while not s_mmt_out.rsp_valid loop
			wait until rising_edge(s_clk);
		end loop;

		wait until rising_edge(s_clk);

		while not s_mmt_out.rsp_valid loop
			wait until rising_edge(s_clk);
		end loop;

		wait until rising_edge(s_clk);

		while not s_mmt_out.rsp_valid loop
			wait until rising_edge(s_clk);
		end loop;

	end process;

end architecture;