library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.common.all;

entity tb_multiplier is
end entity ; -- tb_multiplier

architecture arch of tb_multiplier is
	constant CLK_PERIOD : time := 1 ns;
	
	signal s_clk : std_ulogic := '0';
	signal s_simulation_stop : boolean := false;
	
	signal s_a : std_ulogic_vector(31 downto 0);
	signal s_b : std_ulogic_vector(31 downto 0);
	signal s_q : std_ulogic_vector(63 downto 0);
	signal s_unsigned_op : boolean := false;
	signal s_mixed_op : boolean := false;
begin
	
    ram: entity work.cf_multiplier
		port map (
			i_a => s_a,
			i_b => s_b,
			i_unsigned_op => s_unsigned_op,
			i_mixed_op => s_mixed_op,
			o_q => s_q
		);
	
	process
	begin
		while not s_simulation_stop loop
			s_clk <= not s_clk;
			wait for CLK_PERIOD / 2;
		end loop;
		wait;
	end process;

	process
	begin
		wait until rising_edge(s_clk);

		s_a <= 32x"5";
		s_b <= 32x"5";
		s_unsigned_op <= true;
		s_mixed_op <= false;
		wait until rising_edge(s_clk);

		s_a <= 32x"5";
		s_b <= 32x"4";
		s_unsigned_op <= false;
		s_mixed_op <= false;
		wait until rising_edge(s_clk);

		s_a <= 32x"5";
		s_b <= std_ulogic_vector(to_signed(-4, 32));
		s_unsigned_op <= false;
		s_mixed_op <= false;
		wait until rising_edge(s_clk);

		s_a <= 32x"FFFFFFFF";
		s_b <= 32x"FFFFFFFF";
		s_unsigned_op <= true;
		s_mixed_op <= false;
		wait until rising_edge(s_clk);

		s_a <= 32x"FFFFFFFF";
		s_b <= 32x"FFFFFFFF";
		s_unsigned_op <= false;
		s_mixed_op <= false;
		wait until rising_edge(s_clk);

		wait until rising_edge(s_clk);
		wait until rising_edge(s_clk);

		report "All Tests performed" severity note;
		s_simulation_stop <= true;
		wait;
	end process;

end architecture;