library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
library work;
use work.common.all;

entity cf_gpio is
	port (
		i_clk : in std_ulogic;
		i_clk_enable : in boolean := true;
		i_reset_n : in std_ulogic := '1';
		i_addr : in std_ulogic_vector(0 downto 0);
		i_data : in std_ulogic_vector(31 downto 0);
		i_mask : in std_ulogic_vector(3 downto 0);
		i_read_enable : in boolean;
		i_write_enable : in boolean;
		o_data : out std_ulogic_vector(31 downto 0);
		i_input : in std_logic_vector(31 downto 0);
		o_output : out std_logic_vector(31 downto 0)
	);
end entity; -- cf_gpio

architecture arch of cf_gpio is
	type StateType is record
		output : std_ulogic_vector(31 downto 0);
		data : std_ulogic_vector(31 downto 0);
	end record;

	signal s_r : StateType := (others => (others => '0'));
	signal s_r_next : StateType;
begin

	process (all)
		variable v_r : StateType;
	begin
		v_r := s_r;
		
		if i_read_enable then
			case i_addr is
				
				when "0" =>
					v_r.data := s_r.output;

				when "1" =>
					v_r.data := std_ulogic_vector(i_input);
					
				when others =>
					null;

			end case;
		end if;

		if i_write_enable then
			case i_addr is
				
				when "0" =>
					v_r.output := (s_r.output and not enlarge_mask(i_mask)) or i_data;
					
				when others =>
					null;

			end case;
		end if;
		
		if i_reset_n = '0' then
			v_r.data := (others => '0');
		end if;

		s_r_next <= v_r;
		
		o_data <= s_r.data;
		o_output <= std_logic_vector(s_r.output);
	end process;
	
	process
	begin
		wait until rising_edge(i_clk) and i_clk_enable;
		
		s_r <= s_r_next;
	end process;
	
end architecture;
